import balderhub.auth.lib.scenario_features.user_login_feature

from balderhub.auth.contrib.html.pages.login_page import LoginPage


class UserLoginFeature(balderhub.auth.lib.scenario_features.user_login_feature.UserLoginFeature):
    """
    Implementation of the user login feature for using the :class:`balderhub.contrib.html.pages.LoginPage` HTML pages
    """

    page = LoginPage()

    def insert_username(self, username: str) -> None:
        self.page.input_username.wait_to_be_clickable_for(1).type_text(username, clean_before=True)

    def insert_password(self, password: str) -> None:
        self.page.input_password.wait_to_be_clickable_for(1).type_text(password, clean_before=True)

    def submit_login(self) -> None:
        self.page.btn_login.wait_to_be_clickable_for(1).click()

    def is_already_logged_in(self):
        # TODO improve
        self.page.open()
        return not self.page.is_applicable()



from typing import Union, List

import balderhub.html.lib.scenario_features

import balderhub.html.lib.utils.components as html
from balderhub.url.lib.utils import Url


class LoginPage(balderhub.html.lib.scenario_features.HtmlPage):
    """
    HTML Page for normal login pages as abstract base class - all abstract methods/properties needs to be defined in
    subclass
    """

    @property
    def url(self) -> Url:
        """
        :return: non-schema url the login page is located at
        """
        raise NotImplementedError

    @property
    def applicable_on_url_schema(self) -> Union[Url, List[Url]]:
        return self.url

    def open(self) -> None:
        """
        This method opens the login page.
        """
        self.driver.navigate_to(self.url)

    @property
    def input_username(self) -> html.inputs.HtmlTextInput:
        """
        :return: Html input field where the username needs to be filled
        """
        raise NotImplementedError

    @property
    def input_password(self) -> html.inputs.HtmlPasswordInput:
        """
        :return: Html input field where the password needs to be filled
        """
        raise NotImplementedError

    @property
    def btn_login(self) -> html.HtmlButtonElement:
        """
        :return: HTML button to submit the login form
        """
        raise NotImplementedError



from .base_role_feature import BaseRoleFeature


class UserRoleFeature(BaseRoleFeature):
    """
    config feature that provides a user/password authentification
    """
    @property
    def username(self) -> str:
        """
        :return: the valid username of this user role
        """
        raise NotImplementedError

    @property
    def password(self) -> str:
        """
        :return: the valid password for this user role
        """
        raise NotImplementedError



from .base_role_feature import BaseRoleFeature


class TokenRoleFeature(BaseRoleFeature):
    """
    config feature that provides a basic token as authentication method
    """

    @property
    def token(self) -> str:
        """
        :return: returns the valid token to authenticate
        """
        raise NotImplementedError



from __future__ import annotations
import balder

from balderhub.auth.lib.scenario_features import UserLoginFeature
from balderhub.auth.lib.scenario_features.role import UserRoleFeature


class ScenarioSimpleLogin(balder.Scenario):
    """
    This is a simple scenario that evaluates if a user can be logged in.
    """

    class System(balder.Device):
        """
        the (remote) system that stores the correct credentials
        """
        role = UserRoleFeature()

    @balder.connect('System', over_connection=balder.Connection())
    class Client(balder.Device):
        """
        client device where the login can be executed
        """
        login = UserLoginFeature()

    def test_login(self):
        """
        This test executes a simple login and validates if the login succeeds.
        """
        assert not self.Client.login.is_already_logged_in(), "a user was already logged in"

        self.Client.login.insert_username(self.System.role.username)
        self.Client.login.insert_password(self.System.role.password)

        self.Client.login.submit_login()

        assert self.Client.login.is_already_logged_in(), "login failed because no user has been logged in"



from .base_role_feature import BaseRoleFeature
from .token_role_feature import TokenRoleFeature
from .user_role_feature import UserRoleFeature

__all__ = [
    "BaseRoleFeature",
    "TokenRoleFeature",
    "UserRoleFeature",
]



import balder


class UserLoginFeature(balder.Feature):
    """
    This login feature can be assigned to devices that do provide a basic login handling.
    """

    def insert_username(self, username: str) -> None:
        """
        Inserts the username for login

        :param username: the username to insert
        """
        raise NotImplementedError

    def insert_password(self, password: str) -> None:
        """
        Inserts the password for login

        :param password: the password to insert
        """
        raise NotImplementedError

    def submit_login(self) -> None:
        """
        Executes the login
        """
        raise NotImplementedError

    def is_already_logged_in(self):
        """
        :return: returns True if someone is already logged in, otherwise False
        """
        raise NotImplementedError
